@@ -11,18 +11,18 @@
 #'
 #' - `vars`: Names of local variables (which are created inside the expression).
 #' By default, local variables are automatically detected from the expression
-#' via [codetools::findLocalsList()]. Locally created variables are cached along
-#' with the value of the expression.
+#' via [find_locals()]. Locally created variables are cached along with the
+#' value of the expression.
 #'
 #' - `hash` and `extra`: R objects to be used to determine if cache should be
 #' loaded or invalidated. If (the MD5 hash of) the objects is not changed, the
 #' cache is loaded, otherwise the cache is invalidated and rebuilt. By default,
 #' `hash` is a list of values of global variables in the expression (i.e.,
 #' variables created outside the expression). Global variables are automatically
-#' detected by [codetools::findGlobals()]. You can provide a vector of names to
-#' override the automatic detection if you want some specific global variables
-#' to affect caching, or the automatic detection is not reliable. You can also
-#' provide additional information via the `extra` argument. For example, if the
+#' detected by [find_globals()]. You can provide a vector of names to override
+#' the automatic detection if you want some specific global variables to affect
+#' caching, or the automatic detection is not reliable. You can also provide
+#' additional information via the `extra` argument. For example, if the
 #' expression reads an external file `foo.csv`, and you want the cache to be
 #' invalidated after the file is modified, you may use `extra =
 #' file.mtime("foo.csv")`.
@@ -64,6 +64,8 @@
 #'   loaded and returned (other local variables will also be lazy-loaded into
 #'   the current environment as a side-effect). If cache does not exist, the
 #'   expression is executed and its value is returned.
+#' @references See <https://yihui.org/litedown/#sec:option-cache> for how it
+#'   works and an application in \pkg{litedown}.
 #' @export
 #' @examples
 #' # the first run takes about 1 second
@@ -241,7 +243,8 @@ io_methods = list(
 #' Cache the value of an R expression to an RDS file
 #'
 #' Save the value of an expression to a cache file (of the RDS format). Next
-#' time the value is loaded from the file if it exists.
+#' time the value is loaded from the file if it exists. Please consider using
+#' [cache_exec()] instead, which is more flexible and intelligent.
 #'
 #' Note that the `file` argument does not provide the full cache filename. The
 #' actual name of the cache file is of the form \file{BASENAME_HASH.rds}, where
@@ -320,11 +323,19 @@ io_methods = list(
 #'
 #'   Side-effects (such as plots or printed output) will not be cached. The
 #'   cache only stores the last value of the expression in `expr`.
+#'
+#'   The hash is calculated by saving the hash object to a temporary file via
+#'   `saveRDS()` and getting the MD5 digest of the file. Since the file
+#'   generated by `saveRDS()` may be platform-dependent, the cache may not be
+#'   platform-agnostic. In other words, the cache saved on one platform may be
+#'   invalidated on another platform even if nothing in the hash object has
+#'   changed. By comparison, [cache_exec()] doesn't have this problem.
 #' @return If the cache file does not exist, run the expression and save the
 #'   result to the file, otherwise read the cache file and return the value.
 #' @seealso [cache_exec()], which is more flexible (e.g., it supports in-memory
 #'   caching and different read/write methods for cache files).
 #' @export
+#' @keywords internal
 #' @examples
 #' f = tempfile()  # the cache file
 #' compute = function(...) {
@@ -424,13 +435,36 @@ clean_cache = function(path) {
   unlink(olds[(base == base[keep][1]) & !keep])
 }
 
-# analyze code and find out global variables used from an environment
+#' Find global/local variables in R code
+#'
+#' Use [codetools::findGlobals()] and [codetools::findLocalsList()] to find
+#' global and local variables in a piece of code. Global variables are defined
+#' outside the code, and local variables are created inside the code.
+#' @param code Either a character vector of R source code, or an R expression.
+#' @param envir The global environment in which global variables are to be
+#'   found.
+#' @return A character vector of the variable names. If the source code contains
+#'   syntax errors, an empty character vector will be returned.
+#' @note Due to the flexibility of creating and getting variables in R, these
+#'   functions are not guaranteed to find all possible variables in the code
+#'   (e.g., when the code is hidden behind `eval()`).
+#' @export
+#' @examples
+#' x = 2
+#' xfun::find_globals('y = x + 1')
+#' xfun::find_globals("y = get('x') + 1")  # x is not recognized
+#' xfun::find_globals('y = zzz + 1')  # zzz doesn't exist
+#'
+#' xfun::find_locals('y = x + 1')
+#' xfun::find_locals("assign('y', x + 1)")  # it works
+#' xfun::find_locals("assign('y', x + 1, new.env())")  # still smart
+#' xfun::find_locals("eval(parse(text = 'y = x + 1'))")  # no way
 find_globals = function(code, envir = parent.frame()) {
+  fun = function() {}
   if (is.language(code)) {
-    fun = function() {}
     body(fun) = code
   } else {
-    fun = eval(parse_only(c('function(){', code, '}')), baseenv())
+    fun = eval(parse2(c('function(){', code, '}'), fun), baseenv())
   }
   obj = codetools::findGlobals(fun)
   intersect(obj, ls_all(envir, TRUE))
@@ -452,12 +486,18 @@ ls_all = function(envir, recursive = FALSE) {
   unique(x)
 }
 
-# find local variables in code (those getting assigned in the code)
+#' @rdname find_globals
+#' @export
 find_locals = function(code) {
-  code = if (is.language(code)) as.expression(code) else parse_only(code)
+  code = if (is.language(code)) as.expression(code) else parse2(code)
   codetools::findLocalsList(code)
 }
 
+# if code cannot be parsed, return an empty expression
+parse2 = function(code, fallback = expression()) {
+  tryCatch(parse_only(code), error = function(e) fallback)
+}
+
 # return a list of values of global variables in code
 global_vars = function(code, envir) {
   if (length(vars <- find_globals(code, envir)) > 0)
